package com.bwie.loan.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Repayment;
import com.bwie.common.domain.request.LoanRequest;
import com.bwie.common.domain.response.Loan;
import com.bwie.common.domain.response.UserResponse;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.loan.mapper.LoanMapper;
import com.bwie.loan.service.LoanService;
import org.springframework.amqp.rabbit.connection.CorrelationData;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.UUID;

@Service
public class LoanServiceImpl implements LoanService {

    @Autowired
    private LoanMapper loanMapper;

    @Autowired
    private RepaymentServiceImpl repaymentService;

    @Autowired
    private RedisTemplate<String,String> redisTemplate;

    @Autowired
    private HttpServletRequest request;

    @Autowired
    private RabbitTemplate rabbitTemplate;



    /**
     * 借款数据
     * @return
     */
    @Override
    public Result<List<Loan>> loanData() {
        List<Loan> loanList = loanMapper.loanData();
        //计算已还本金和未还本金
        if (loanList.size()>0){
            for (Loan loan : loanList) {
                //获取当前借款还款计划
                Result<List<Repayment>> listResult = repaymentService.repaymentListByLoanId(loan.getId());
                List<Repayment> repaymentList = listResult.getData();
                //已还本金
                double repaidPrincipal=0.0;
                //未还本金
                double outstandingPrincipal=loan.getLoanAmount();
                //每期还款本金
                double avgMoney = loan.getLoanAmount()/loan.getPeriods();
                //对还款计划集合进行处理
                if (repaymentList.size()>0){
                    for (Repayment repayment : repaymentList) {
                        if (repayment.getStatus()==1) {
                            repaidPrincipal += avgMoney;
                        }
                    }
                }
                //把未还本金和已还本金添加到借款对象中
                loan.setOutstandingPrincipal(outstandingPrincipal-repaidPrincipal);
                loan.setRepaidPrincipal(repaidPrincipal);
            }
        }
        return Result.success(loanList);
    }

    /**
     * 借款
     * @param loanRequest
     * @return
     */
    @Override
    @Transactional
    public Result insertLoan(LoanRequest loanRequest) {
        //获取当前登录人
        String token = request.getHeader("token");
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        UserResponse userResponse = JSON.parseObject(s, UserResponse.class);
        loanRequest.setCreateBy(userResponse.getId());
        //判断利率
        if (loanRequest.getLoanAmount()<=2000){
            loanRequest.setLoanInterest(Double.valueOf(redisTemplate.opsForValue().get("ONE")));
        }else if (loanRequest.getLoanAmount()<=20000){
            loanRequest.setLoanInterest(Double.valueOf(redisTemplate.opsForValue().get("TWO")));
        }else if (loanRequest.getLoanAmount()<=50000){
            loanRequest.setLoanInterest(Double.valueOf(redisTemplate.opsForValue().get("THREE")));
        }else if (loanRequest.getLoanAmount()<=100000){
            loanRequest.setLoanInterest(Double.valueOf(redisTemplate.opsForValue().get("FOUR")));
        }else {
            loanRequest.setLoanInterest(Double.valueOf(redisTemplate.opsForValue().get("FIVE")));
        }
        //添加数据到数据库
        loanMapper.insertLoan(loanRequest);
        //新增加借款主键id
        Integer id = loanRequest.getId();
        //异步发送消息
        rabbitTemplate.convertAndSend("ONE_QUEUE",id.toString(),message -> {
            message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
            return message;
        });
        return Result.success();
    }

    /**
     * 根据借款id查询接口
     * @param loanId
     * @return
     */
    @Override
    public Loan findLoanById(Integer loanId) {
        Loan loan = loanMapper.findLoanById(loanId);
        return loan;
    }

    /**
     * 修改为未放款状态
     * @param id
     */
    @Override
    public void updateStatus(String id,Integer status) {
        loanMapper.updateStatus(id,status);
    }
}
